
fitgene<-function(node,mutationmat,tumors,pl=F,title="",ymax=0.25, densscale=100, densat=c(0.1,0.2)){
	count<-rep(NA,max(tumors))
	for(i in 1:max(tumors)){
		temptumor<-which(tumors<=i)
		if(length(temptumor)>0) {
			count[i]<-length(which(mutationmat[temptumor,node]==1))/length(temptumor)
		}
		else {
			count[i]<-0
		}
	}
	
	t<-(1:length(count))
	para<-list(slop=5,infl=20)
	if(count[length(count)]==0) {
		return(c(0,50))
	}
	
	if(ymax<max(count)) {
		ymax<-max(count)+0.01
	}
	densscale<-densscale*ymax
	densat<-ymax/c(2.2,1.1)
	w<-density(tumors,from=1,to=length(count),n=length(count))
	
	tems<-nls(count~(count[length(count)])/(1+exp((infl-t)/slop)),control=nls.control(minFactor=1/2^10,warnOnly=T,maxiter=1000),start=para,lower=list(slop=1,infl=1),upper=list(slop=700,infl=1500),algorithm="port",trace=F,weights=w$y)
	if(pl) {
		plot(t,count,xlim=c(0,max(tumors)),ylim=c(0,ymax),main="",xlab="",ylab="" )
		title(title,line=-1)
		lines(t,fitted.values(tems),lwd=4,col="red")
		lines(t,w$y*densscale,col="darkgrey",lwd=2)
		axis(4,at=densat,labels=round((densat/densscale),3),col.axis="darkgrey",cex=0.8)
		
	}
	return(c(summary(tems)$parameters[1,1],summary(tems)$parameters[2,1]))
}

load(file="~/ubinAMPLOSS.RData")
ubinMAT<-ubinAMPLOSS
N<-dim(ubinMAT)[1]

tumors<-apply(ubinMAT,1,sum)

#only fit if more than 8% is affected i.e. 45 // 6% i.e. 34
min_affected<-floor(N*0.06)

par_i=NULL;
par_s=NULL;

fitgenelist<-NULL
for( i in 1:dim(ubinMAT)[2]) {
	
	if(sum(ubinMAT[,i])>min_affected) {
		res<-try(fitgene(i,ubinMAT,tumors),silent=T)
		if(class(res)=="try-error"){
			print("TTRRYY error")
			print(i)
		}
		
		if(class(res)!="try-error"){
			par_s=rbind(par_s,res[1])
			par_i=rbind(par_i,res[2])
			fitgenelist=rbind(fitgenelist,i)
		}
		else {
			par_s=rbind(par_s,700)
			par_i=rbind(par_i,800)
		}
	}
	else {
		par_s=rbind(par_s,700)
		par_i=rbind(par_i,800)
	}
}

print(colnames(ubinMAT[,c(order(par_s)[1:10])]))


###significance analysis (might take very long if done for all genes (mx)!)
mx<-length(fitgenelist)
l = 0
p_values=vector()
tumorpi=tumors/dim(ubinMAT)[2]
tumorpi=tumorpi/sum(tumorpi)
for(i in order(par_s)[1:mx]) {
	l=l+1
	background_par_s=list()
	for(j in 1:1000) {
		newmut=rep(0,dim(ubinMAT)[1])
		tumorpi_it=tumorpi
		for(k in 1:sum(ubinMAT[,i])) {
			newmut=newmut+rmultinom(1,1,tumorpi_it)
			tumorpi_it=tumorpi_it*(!newmut)
		}
		addubinMAT=cbind(ubinMAT,newmut)
		tumorsadd<-apply(addubinMAT,1,sum)
		res<-try(fitgene(dim(addubinMAT)[2],addubinMAT,tumorsadd,pl=F),silent=T)
		background_par_s[j]=res[1]
	}
	p_values[l]=sum(background_par_s<=par_s[i])
}
print(cbind(colnames(ubinMAT)[order(par_s)[1:mx]],(p_values[1:mx])/1000,p.adjust(p_values,method="fdr")))

